@@ -13,6 +13,34 @@ symmetry operations in symbolic form.
 The module also includes conversion tools between ASE and spglib formats,
 as well as symmetry naming utilities.
 
+Classes
+-------
+Cell
+    Wrapper around ASE Atoms + spglib tuple (lattice, positions, numbers).
+    Provides:
+    - from_file(path): Read a file and return Cell.
+    - from_spglib_tuple(tup): Construct Cell from spglib-format tuple.
+    - get_sym_info(symprec=...): Print symbolic symmetry operations and Wyckoff info.
+    - get_wyckoff_positions(symprec=...): Group atoms by Wyckoff positions.
+    - get_supercell(supercell): Return a repeated Cell object.
+    - write_espresso_in(...): Write Quantum ESPRESSO input file.
+
+Functions
+---------
+ase2spglib(crystal_ase)
+    Convert an ASE Atoms object to a spglib tuple (lattice, positions, numbers).
+
+spglib2ase(spglib_crystal)
+    Convert a spglib-format tuple back into an ASE Atoms object.
+
+read_spg(file)
+    Read a structure file and return its spglib-compatible tuple.
+
+Private Utilities
+-----------------
+_rot_name(rot, lattice)
+    Identify the symbolic name and axis of a rotation matrix (e.g. 'C3', 'm') using its eigenstructure.
+
 Examples
 --------
 >>> from yaiv.cell import Cell
@@ -33,6 +61,8 @@ yaiv.utils    : Utility functions for basis and vector transformations
 """
 
 from types import SimpleNamespace
+import re
+import os
 
 import numpy as np
 import spglib as spg
@@ -41,6 +71,7 @@ from ase.io import read, write
 from ase import Atoms
 
 from yaiv.defaults.config import defaults as dft
+from yaiv.defaults.config import qe_defaults
 from yaiv import utils as ut
 
 __all__ = ["read", "write", "read_spg", "ase2spglib", "spglib2ase", "Cell"]
@@ -61,6 +92,21 @@ class Cell:
     spglib : tuple
         Tuple (lattice, positions, numbers) derived from the Atoms object,
         used for spglib symmetry operations.
+
+    Methods
+    -------
+    from_file(...)
+        Read a structure file using ASE and return a Cell instance.
+    from_spglib_tuple(...)
+        Initialize from a (lattice, positions, numbers) spglib tuple.
+    get_sym_info(...)
+        Print a detailed report of symmetry information for the crystal structure.
+    get_wyckoff_positions(...)
+        Analyze the structure and store information about independent Wyckoff positions.
+    get_supercell(...)
+        Construct a supercell by repeating the current unit cell along each lattice direction.
+    write_espresso_in(...)
+        Write Quantum ESPRESSO input file using either default parameters or a template.
     """
 
     def __init__(
@@ -100,7 +146,7 @@ class Cell:
             )
         else:
             self.spglib = (np.array(lattice), np.array(positions), np.array(numbers))
-            self.atoms = spglib2ase(atoms)
+            self.atoms = spglib2ase(self.spglib)
 
     @classmethod
     def from_file(cls, file: str):
@@ -187,7 +233,7 @@ class Cell:
         print("Site symmetry simbols:")
         print(dataset.site_symmetry_symbols)
         print()
-        print("SYMMETRY OPERATIONS:")
+        print("Symmetry Operations:")
         print()
         symmetry = [(r, t) for r, t in zip(dataset.rotations, dataset.translations)]
         for i in range(len(symmetry)):
@@ -269,6 +315,136 @@ class Cell:
             indices=grouped_indices,
         )
 
+    def get_supercell(self, supercell: list[int] = [1, 1, 1]) -> "Cell":
+        """
+        Construct a supercell by repeating the current unit cell along each lattice direction.
+
+        This method generates a new `Cell` object representing a supercell formed by tiling
+        the original cell along the three lattice vectors.
+
+        Parameters
+        ----------
+        supercell : list[int], optional
+            A list of 3 integers specifying how many times to replicate the cell along
+            each lattice direction. Default is [1, 1, 1] (no replication).
+
+        Returns
+        -------
+        Cell
+            A new `Cell` object representing the expanded supercell.
+        """
+
+        lattice = np.copy(self[0])  # Original lattice vectors (3x3)
+        positions_list = []
+
+        for i in range(supercell[0]):
+            for j in range(supercell[1]):
+                for k in range(supercell[2]):
+                    displacement = np.array([i, j, k])
+                    new_pos = self[1] + displacement  # Shift all atoms
+                    positions_list.append(new_pos)
+
+        # Stack all positions and flatten to a single (N_atoms × 3) array
+        positions = np.vstack(positions_list)
+
+        # Repeat the atomic elements accordingly
+        elements = np.tile(self[2], np.prod(supercell))
+
+        # Expand the lattice
+        for i in range(3):
+            lattice[i] *= supercell[i]
+            positions[:, i] /= supercell[i]  # Normalize positions to new cell
+
+        return Cell(lattice, positions, elements)
+
+    def write_espresso_in(self, filename: str = "espresso.pwi", template: str = None):
+        """
+        Write Quantum ESPRESSO input file using either default parameters or a template.
+
+        If no template is provided, writes a new input using default settings stored
+        in `qe_defaults`. If a template is provided, it replaces the structural
+        information (cell, atomic positions, nat) in the template with those from
+        `self.atoms`.
+
+        Parameters
+        ----------
+        filename : str
+            Output input file name (e.g. 'espresso.pwi').
+        template : str
+            Optional template input file to use as a base. Only geometry-related fields
+            (CELL_PARAMETERS, ATOMIC_POSITIONS, nat) are updated.
+        """
+        if template is None:
+            write(
+                filename,
+                self.atoms,
+                input_data=qe_defaults.input_data,
+                kpts=qe_defaults.kpts,
+                format="espresso-in",
+            )
+            return
+
+        # Write a temporary QE input from the structure to extract updated geometry
+        write(".tmp.pwi", self.atoms, format="espresso-in")
+
+        # Extract updated geometry info: CELL_PARAMETERS, ATOMIC_POSITIONS, nat
+        basis = []
+        pos = []
+        cell_line = -4
+        pos_line = -999999
+        nat = 0
+        with open(".tmp.pwi", "r") as lines:
+            for n, line in enumerate(lines):
+                if re.search(r"\bnat\b", line):
+                    nat = int(line.split()[2])
+                if re.search("CELL_PARAMETERS", line):
+                    cell_line = n
+                if re.search("ATOMIC_POSITIONS", line):
+                    pos_line = n
+                if n - cell_line in [1, 2, 3]:
+                    basis.append(line)
+                if n - pos_line in range(1, nat + 1):
+                    pos.append(line)
+        os.remove(".tmp.pwi")
+
+        # Open template and inject updated structural info
+        write_nat = True
+        write_pos = True
+        write_basis = True
+        K_points = False
+
+        temp = open(template, "r")
+        output = open(filename, "w")
+        for line in temp:
+            if re.search("ibrav", line):
+                if "0" not in line:
+                    raise ValueError("ERROR: Your template must have ibrav = 0.")
+            elif re.search("pseudo_dir", line):
+                line = "  pseudo_dir = '$PSEUDO_DIR',\n"
+            elif re.search("outdir", line):
+                line = "  outdir = './tmp',\n"
+            elif re.search("nat*=", line) and write_nat == True:
+                line = "  nat=" + str(nat) + ",\n"
+                write_nat = False
+            elif re.search("POSITIONS", line, re.IGNORECASE) and write_pos == True:
+                line = "ATOMIC_POSITIONS {angstrom}\n"
+                output.write(line)
+                for line in pos:
+                    output.write(line)
+                write_pos = False
+            elif re.search("POINTS", line, re.IGNORECASE):
+                K_points = True
+            elif re.search("CELL", line, re.IGNORECASE):
+                line = "CELL_PARAMETERS {angstrom}\n"
+                output.write(line)
+                for line in basis:
+                    output.write(line)
+                K_points = False
+            if write_pos == True or K_points == True:
+                output.write(line)
+        temp.close()
+        output.close()
+
 
 def ase2spglib(crystal_ase: Atoms) -> tuple:
     """
@@ -291,7 +467,7 @@ def ase2spglib(crystal_ase: Atoms) -> tuple:
         - positions : (N, 3) array of scaled atomic positions (fractional)
         - numbers : (N,) array of atomic numbers
     """
-    lattice = np.array(crystal_ase.get_cell())
+    lattice = np.asarray(crystal_ase.get_cell()).copy()
     positions = crystal_ase.get_scaled_positions()
     numbers = crystal_ase.get_atomic_numbers()
     spglib_crystal = (lattice, positions, numbers)
